
var BadPerson = function (maingame, game, from, to, speed) {

    this.maingame = maingame;
    this.game = game;

    this.bullets = this.maingame.status.groups.enemyBullets;

    Phaser.Sprite.call(this, this.game, from.x, from.y, 'anim_badppl', 0);
    this.anchor.setTo(0.45, 0.46);
    this.game.add.existing(this);

    this.settings = {
        waypoints: [],
        currenttarget: 1,
        movementtween: null,
        statustween: null,
        speed: speed,
        status: "normal",
        statussmilee: null,
        smileeduration: 3000,
        fieldofview: 80,
        timetoshoot: 750,
        timebetweenshots: 750,
        pacified: false,
        shots: []
    }

    this.sounds = {
        shot: this.game.add.audio('snd_shot', 1, false),
        grunt: this.game.add.audio('snd_grunt', 1, false)
    }

    this.settings.waypoints[0] = from;
    this.settings.waypoints[1] = to;

    this.settings.statussmilee = this.game.add.sprite(0, 0, "smilee");
    this.settings.statussmilee.scale = {x: 0.35, y: 0.35};
    this.settings.statussmilee.alpha = 0;
    this.settings.statussmilee.anchor.setTo(0.5, 0.5);

}

BadPerson.prototype = Object.create(Phaser.Sprite.prototype);
BadPerson.prototype.constructor = BadPerson;

/**
 * Automatically called by World.update
 */
BadPerson.prototype.update = function() {


};


BadPerson.prototype.pointTo = function(point) {
    var dist = new Phaser.Point(this.x - point.x, this.y - point.y);
    var angle = Math.atan2(dist.y, dist.x) * (180 / Math.PI);
    this.angle =  (angle - 90) % 360;
};


BadPerson.prototype.moveTo = function(targetid) {

    var target = this.settings.waypoints[targetid];

    //console.log("moveTo", target,this.settings.waypoints, targetid);

    // get distance for speed calculation
    var dist = new Phaser.Point(this.x - target.x, this.y - target.y);

    // rotate in direction of movement
    //var targetAngle = this.game.math.angleBetween(
    //    this.x, this.y,
    //    target.x, target.y
    //);

    this.pointTo(target);

    this.settings.movementtween = this.game.add.tween(this).to(
        {x: target.x, y: target.y},
        (dist.getMagnitude() / 100) * this.settings.speed,
        Phaser.Easing.Linear.None,
        true,
        0,
        0,
        false);

    var that = this;
    this.settings.movementtween.onComplete.add(function() {

        var thot = that;
        //console.log("CT: ", thot.settings.currenttarget);

        if (thot.settings.currenttarget == 1) {
            thot.settings.currenttarget = 0;
        } else {
            thot.settings.currenttarget = 1;
        }
        //console.log("DO: ", thot.settings.currenttarget);
        thot.moveTo(thot.settings.currenttarget);
    });

    //console.log(this.settings.tween);

}

BadPerson.prototype.start = function() {

    if (this.settings.waypoints[1] != null) {
        this.moveTo(1);
    }

};


BadPerson.prototype.smileeHit = function() {

    if (this.settings.pacified)
        return;

    this.settings.movementtween.pause();
    this.settings.statussmilee.x = this.x;
    this.settings.statussmilee.y = this.y;
    this.settings.statussmilee.alpha = 1;

    this.settings.status = "confused";

    this.settings.statustween = this.game.add.tween(this).to(
        {angle: 360},
        this.settings.smileeduration,
        Phaser.Easing.Linear.None,
        true,0,0,false);

    var that = this;
    this.settings.statustween.onComplete.add(function() {
        var thot = that;
        thot.settings.status = "normal";
        thot.settings.statussmilee.alpha = 0;
        thot.settings.statustween.stop();
        thot._resumeWalking();
    });

}

BadPerson.prototype.bigHit = function() {

    if (this.settings.status == "confused") {
        this.settings.status = "pacified";
        this.settings.pacified = true;
        this.animations.frame = 1;
        this.settings.statustween.pause();
        this.settings.statustween.stop();
        this.settings.movementtween.stop();
    }

}


BadPerson.prototype._canSeePlayer = function(player) {

    this.player = player;

    var to    = new Phaser.Point( player.x, player.y );
    var from  = new Phaser.Point( this.x, this.y );
    var dist  = new Phaser.Point( to.x - from.x, to.y - from.y );

    var distangle = ( Math.atan2(dist.y, dist.x) * (180 / Math.PI) );
    var viewangle = this.angle-90;

    if (distangle<0) {distangle += 360;}
    if (viewangle<0) {viewangle += 360;}

    var anglebetween = 180 - Math.abs(Math.abs(distangle - viewangle) - 180);

    //console.log("a: ", distangle, "v: ", viewangle );
    //console.log(anglebetween);

    if ( (anglebetween*2) > this.settings.fieldofview) {
        return false;
    }

    // can see him theoretically lets check line of sight
    var tilehits = this.maingame._raycast({worldX: this.world.x, worldY: this.world.y});

    if (tilehits.length > 0)
        return false;

    return true;

};


BadPerson.prototype._shootPlayer = function(player) {

    //console.log(this.settings.status);

    if (this.settings.pacified || this.settings.status == "confused")
        return;

    if (this._canSeePlayer(this.player) && this.bullets.countDead()>0 ) {

        this.sounds.shot.play();

        var bullet = this.bullets.getFirstDead();

        bullet.reset(this.x, this.y);

        bullet.rotation = this.game.physics.arcade.moveToObject(bullet, player, 500) + (Math.PI/2);

        this.pointTo( player );

        // prepare new shot
        var that = this;
        setTimeout(function(){
            var thot = that;
            thot._shootPlayer(player);
        }, this.settings.timebetweenshots);
    } else {
        this._resumeWalking();
    }

};

BadPerson.prototype._foundPlayer = function(player) {

    if (this.settings.pacified)
        return;

    this.pointTo(player);

    this.settings.status = "preparing";

    this.settings.movementtween.pause();

    this.sounds.grunt.play();

    var that = this;
    setTimeout(function(){
        var thot = that;
        thot._shootPlayer(player);

    }, this.settings.timetoshoot);

};


BadPerson.prototype._resumeWalking = function() {
    this.settings.status = "normal";
    this.settings.movementtween.resume();
    this.pointTo(this.settings.waypoints[this.settings.currenttarget]);
};


module.exports = BadPerson;
